***************************************************************************
*                                                                         *
*  File    : Qsudfs.prg                                                   *
*  Date    : January 27, 1988                                             *
*  By      : Ray Kiuchi and Pedro Chiang                                  *
*  Version : 1.00                                                         *
*  Purpose : A collection of user defined functions for QuickSilver 1.0   *
*            and later.                                                   *
*  Notes   : Needs to be used in conjunction with QSEXT.LIB if the UDF    *
*            issues a CCALL.                                              *
*                                                                         *
*            To link with your QS application use the following procedure *
*                                                                         *
*            1. Include the line SET UDF TO QSUDFS in your program file.  *
*            2. At QS time, QS -c,QSEXT.LIB <additional switches> <prg>.  *
*                                                                         *
*  Table of                                                               *
*  Contents:  1. Header(), Fldcount(), Isunique(), Ndxkey(), Ndxno(),     *
*             6. Order(), Dirmake(), Cursor(), Todollars(), Match(),      *
*            11. Expand(), Dup(), Pmt(), Futureval(), Presentval(),       *
*            16. Pow(), Root(), Ampm(), Arrsave(), Arrsort(),             *
*            21. Arrcheck(), Dtow(), Julian(), Fromjul(), Rat()           *
*            26. Seconds()                                                *
*                                                                         *
*  Last                                                                   *
*  update  : February 12, 1988                                            *
*                                                                         *
***************************************************************************


* Function : Header
* Purpose  : Return <Exp num> the number of bytes of the header file.

FUNCTION Header
private wt_size

wt_size=0
ccall header with wt_size
return (wt_size)

* Eof: Header


************
*
* Function : Fldcount
* Purpose  : Return <Exp num> the number of fields in the current work area.

FUNCTION Fldcount
private wt_flds

wt_flds=0
ccall header with wt_flds
return (int(wt_flds / 32) - 1)

* Eof: Fldcount


************
*
* Function : Isunique
* Purpose  : Return <Exp log> whether the current index was created using UNIQUE.

FUNCTION Isunique
private wt_unik

wt_unik=.f.
ccall isunique with wt_unik
return (wt_unik)

* Eof: Isunique


************
*
* Function : Ndxkey
* Purpose  : Return <Exp char> index key of the current index.

FUNCTION Ndxkey
private wt_key

wt_key = space(100)
ccall ndxkey with wt_key
return (wt_key)

* Eof: Ndxkey


************
*
* Function : Ndxno
* Purpose  : Return <Exp num> number of indexes set on the current work area.

FUNCTION Ndxno
private wt_ndxno

wt_ndxno=0
ccall ndxno with wt_ndxno
return (wt_ndxno)

* Eof: Ndxno


************
*
* Function : Order
* Purpose  : Return <Exp num> number of active index.

FUNCTION Order
private wt_ord

wt_ord=0
ccall order with wt_ord
return (wt_ord)

* Eof: Order


************
*
* Function : Dirmake(<exp char>, <exp char>)
* Purpose  : Create a directory
*
* 1. The first one is the directory name to create.
*    A path can be specified, otherwise it will be created in the
*    current directory.
* 2. The second one is the return code and it returns 3 different values.
*    - 0  successful
*    - 3  wrong dos path
*    - 5  directory already exits

FUNCTION Dirmake
parameters wt_dir, wt_error

ccall mkdir with wt_dir, wt_error
return []

* Eof: Dirmake


************
*
* Function : Cursor(<Exp char>)
* Purpose  : Turn the cursor off/on

FUNCTION Cursor
parameters wt_sw
private wt_attr

wt_sw = upper(wt_sw)
if wt_sw = 'OFF'
   wt_attr = 5120
else
   wt_attr = iif(iscolor(),1543,3085)
endif

wt_int=16
wt_ax=256
wt_bx=0
dosint wt_int,wt_ax,wt_bx,wt_attr

return []

* Eof: Cursor


************
*
* Function : Todollars
* Purpose  : Return <Exp char> from <exp num> : dollars ... cents
* Note     : Maximum lenght are 7 digits

FUNCTION Todollars
parameter num
private word, aa, bb, cc, cents, strnum, numele, a, i, j

if num > 9999999.99
   return ([******.**])                       &&| overflow
endif

num=abs(num)
strnum=ltrim(str(num,10,2))
cents=right(strnum,2)
numele=len(strnum)-3

word = []
declare units[10], teens[10], tys[10]
aa = "one       two       three     four      five      six       seven     eight     nine       "
cc = "ten       eleven    twelve    thirteen  fourteen  fifteen   sixteen   seventeen eighteen  nineteen   "
bb = "          twenty    thirty    forty     fifty     sixty     seventy   eighty    ninety               "

for i=0 to 9
    units[i+1]=trim(substr(aa,(i*10)+1,10))
    teens[i+1]=trim(substr(cc,(i*10)+1,10))
      tys[i+1]=trim(substr(bb,(i*10)+1,10))
next

declare array[numele]
for i=numele to 1 step -1
    array[i] = substr(strnum,(numele-i)+1,1)
next i
for i=numele to 1 step -1
    if array[i] <> '0'
       a = match()
    endif
next i
word = word + ' dollars and ' + cents + ' cents.'

return (word)

* Eof: Todollars


************
*
* Function : Match
* Purpose  : Return <Exp char>
* Note     : Called by Todollars.

FUNCTION Match

do case
   case (i=5 .or. i=2)                             &&| ten thousands | tens
     word = word + ' ' + iif((array[i]+array[i-1] >= '10' .and.;
            array[i]+array[i-1] < '20'),(teens[val(array[i-1])+1] +;
     iif(i=5, ' thousand',  "")),tys[val(array[i])] +;
     iif(array[i-1] = '0' .and. i<>2,' thousand', ""))

     array[i-1] = iif((array[i]+array[i-1] >= '10' .and.;
                  array[i]+array[i-1] < '20'),'0', array[i-1])

   case i = 6                                     &&| hundred thousands
    word = word + ' ' + units[val(array[i])] +;
    iif((array[i-1]='0' .and. array[i-2]='0'), ' hundred thousand', ' hundred')

   otherwise
    word = word + ' ' + units[val(array[i])] +;
    iif(i<=3, iif(i=1, "", ' hundred'), iif(i=4, ' thousand', ' million'))
endcase

return []

* Eof: Match


************
*
* Function : Expand
* Purpose  : Return <Exp char> insert n spaces between characters.

FUNCTION Expand
parameters strin, inbet

strout = []
strlen = len(strin)
declare array[strlen]

for i=1 to strlen
    array[i]=substr(strin,i,1)
    strout = strout + array[i] + space(inbet)
next i

return (strout)

* Eof: Expand


************
*
* Function : Dup
* Purpose  : Looks for duplicate in lookup file.
* Notes    : Usefull when used in @ GET VALID
* Syntax   : @ SAY GET memvar VALID dup(memvar, '<1 ... 10>')

FUNCTION Dup
parameter wt_var, wt_area
private wt_curr

wt_curr = ltrim(str(select()))
select &wt_area
seek wt_var
select &wt_curr

if found()

   return (.t.)
else
   return (.f.)
endif

* Eof: Dup


************
*
* Function : Pmt
* Purpose  : Returns mortgage payment per period.
* Syntax   : PMT( <principal>, <interest rate per period>, <periods>)


FUNCTION Pmt
parameter wt_pc, wt_int, wt_pds

return ( wt_pc * ( wt_int / (1.00 - pow((1.00+wt_int),-wt_pds))) )

* Eof: Pmt


************
*
* Function : Futureval
* Purpose  : Returns the future value of a series of cash flows.
* Syntax   : Futureval(<payment>, <rate>, <periods>)

FUNCTION Futureval
parameters wt_pmt, wt_rate, wt_pds

return (wt_pmt * (pow((1.00+wt_rate), wt_pds) -1.00)/wt_rate)

* Eof: Futureval


************
*
* Function : Presentval
* Purpose  : Returns the present value of a series of equal pmts.
* Syntax   : Presentval(<payment>, <rate>, <periods>)

FUNCTION Presentval
parameters wt_pmt, wt_rate, wt_pds

return (wt_pmt * ((1.00 - pow((1+wt_rate),-wt_pds))/wt_rate))

* Eof: Presentval


************
*
* Function : Pow
* Purpose  : Returns the value obtained by raising the value of <x> by <y>.
* Syntax   : POW(<x>,<y>)

FUNCTION Pow
parameters wt_base, wt_power

if wt_base < 0
   return -exp( wt_power * log(abs(wt_base)) )
else
   return  exp( wt_power * log(abs(wt_base)) )
endif

* Eof: Pow


************
*
* Function : Root
* Purpose  : Returns nth root of number
* Syntax   : Root(<x>,<n>)

FUNCTION Root
parameters wt_base, wt_power

wt_power = (1.00/wt_power)

if wt_base < 0
   return -exp( wt_power * log(abs(wt_base)) )
else
   return  exp( wt_power * log(abs(wt_base)) )
endif

* Eof: Root


************
*
* Function : Ampm
* Purpose  : Returns time in am pm format
* Syntax   : Ampm()

FUNCTION Ampm

if time() >= '00:00:00' .and. time() <= '11:59:59'                && am
   tt= iif (time() >= '00:00:00' .and. time() <= '00:59:59',;
       str((val(left(time(),2))+12),2,0) + right(time(),6) + " am",;
       iif(left(time(),1)='0', " " + right(time(),7) + " am", time() + " am"))


else
   tt= iif(time() >= '13:00:00',;
       str((val(left(time(),2))-12),2,0) + right(time(),6) + " pm",;
       time() + ' pm')
endif

return (tt)

* Eof: Ampm


************
*
* Function : Arrsave
* Purpose  : Saves a QS - dBXL array to disk
* Syntax   : Arrsave(<target file>,<source array>)

FUNCTION Arrsave
parameters wt_file, wt_arr
private wt_res

wt_file=wt_file+'.arr'
save all like &wt_arr to &wt_file
wt_res = iif(file('&wt_file'),.t.,.f.)

return (wt_res)

* Eof: Arrsave


************
*
* Function : Arrsort
* Purpose  : Sorts a (C,N or D) QS - dBXL array.
*          : Puts the sorted array onto WT_TEMP[]. 
* Syntax   : Arrsort(<number of elements>).
*          : returns .T. when successfull .F. when not.
* 
* NOTES    : To use this function use the following steps:
*
*            Copy the source array to the target array called
*            "WT_TEMP[]" by issuing:
*            
*                 declare wt_temp[< # of elements of the source array >]
*            
*                 for x=1 to <total # of elements of the source array>
*                     wt_temp[x] = source_array[x]
*                 next x    
*                 
*                 * checking that all elements are the same type.
*                 
*                 if arrcheck(<# of elem. of the source array>)    
*                    a = arrsort(<# of elem.>)     
*                 endif    

FUNCTION Arrsort
parameters xn

private i,j,gap,xtemp,isok
isok = .t.
if type("wt_temp[1]") = 'L'
   return (.not. isok)
endif   

for gap = xn to 1 step -(gap/2)
    gap = int(gap)
    for i=gap to xn                         
        j = i - gap
        do while (j > 0)
           if (wt_temp[j] < wt_temp[j+gap])
              exit
           endif   
           xtemp = wt_temp[j]
           wt_temp[j] = wt_temp[j+gap]
           wt_temp[j+gap] = xtemp
           j = j - gap
        enddo     	  
    next i
next gap    

return ( isok )

* Eof: Arrsort


************
*
* Function : Arrcheck 
* Purpose  : Checks to see whether all the elements are of the same type (CLDN)
* Syntax   : Arrcheck(<number of elements>)
* NOTES    : To use this function use the following steps:
*

FUNCTION Arrcheck
parameters xn

private i, notok, check
notok = .f.
check = type("wt_temp[1]")

for i = 2 to xn
    if check <> type("wt_temp[i]")
       return ( notok )
    endif
next i

return (.not. notok )       

* Eof: Arrcheck


************
*
* Function : Dtow 
* Purpose  : Returns <Exp char> from a given date.
* Syntax   : Dtow(<date type>)
* Example  : Dtow(date()) -->  Thursday, August 11, 1988
*

FUNCTION Dtow
parameters wt_date

declare month[12], days[12]

year="January  February March    April    May      June     July     August   SeptemberOctober  November December "
week="Sunday   Monday   Tuesday  WednesdayThursday Friday   Saturday                                              "

for i=0 to 11
    month[i+1] = trim(substr(year,i*9+1,9))
    days[i+1] = trim(substr(week,i*9+1,9))
next

wt_month=month[month(wt_date)]
wt_day=days[dow(wt_date)]
wt_year=str(year(wt_date),4,0)
                                                                          
return (wt_day + ", " + wt_month + " " + right(dtos(wt_date),2) + ", " + wt_year)

* Eof: Dtow    


************
*
* Function : Julian 
* Purpose  : Returns <Exp num> in julian format from a given date.
* Syntax   : Julian(<date type>)
* Example  : Julian(ctod('02/15/87')) -->  2442824
*

FUNCTION Julian
parameters wt_cal

wt_cal = right(dtos(wt_cal),6)
wt_month = val(substr(wt_cal,3,2))
wt_day = val(substr(wt_cal,5,2))
wt_year = val(left(wt_cal,2))

if wt_month > 2
   m = wt_month + 1
   y = wt_year
else
   m = wt_month + 13
   y = wt_year - 1
endif

return int(floor(365.25 * (1900 + y)) + floor(30.6001 * m) + wt_day + 1720982)      

* Eof: Julian
             

************
*
* Function : Fromjul 
* Purpose  : Returns <Exp date> from a julian date.
* Syntax   : Fromjul(<Exp num>)
* Example  : Fromjul(2446824) -> '01/28/87' date type
*

FUNCTION Fromjul
parameter wt_jul

wt_day = wt_jul - 1720982
y = floor((wt_day - 122.1)/365.25)
m = floor((wt_day - floor(365.25 * y))/30.6001)
d = wt_day - floor(365.25 * y) - floor(30.6001 * m)
m = iif((m < 14), m - 1, m - 13)
y = iif((m < 3), y + 1, y)
    
inter = str(int((y - 1900) * 10000.0 + m * 100.0 + d),6,0)

return ctod(substr(inter,3,2) + "/" + right(inter,2) + "/" + left(inter,2))

* Eof: Fromjul
 

************
*
* Function : Rat  -->  right AT()
* Purpose  : Returns <Exp num> position in a <exp chr2) where a specified
*            substring <exp chr1> begins. 
* Syntax   : Rat(<Exp chr1>,<Exp chr2>)
* Example  : rat('oops','oopsyeahoops') --> 9
*

FUNCTION Rat
parameters sstr,tstr

lent = len(tstr)
lens = len(sstr)
declare s[lens], t[lens]

do while lent >= lens
   lent=len(tstr)
   nl = lent-lens
   equal = 0

   for i=1 to lens
       s[i] = substr(sstr,i,1)
       t[i] = substr(tstr,nl+i,1)
       if s[i] <> t[i]
          break
       else
          equal = equal + 1
          if equal = lens
             return (nl+1)
          endif
       endif
   next

   tstr = left(tstr,lent-1)
enddo
return [0]

* Eof: Rat


************
*
* Function : Seconds
* Purpose  : Returns <Exp num> of time in seconds
* Syntax   : Seconds()
*

FUNCTION Seconds

wt_time=time()

return int(val(left(wt_time,2))*3600 + val(substr(wt_time,4,2))*60 +;
        val(right(wt_time,2)))

* Eof: Seconds
